@@ -6,6 +6,7 @@ grid.
 """
 
 import json
+import logging
 from typing import Any, Dict, Iterator, List, Optional, Sequence, Union
 
 import numpy as np
@@ -28,6 +29,7 @@ class FieldCollection(FieldBase):
         *,
         copy_fields: bool = False,
         label: Optional[str] = None,
+        labels: List[Optional[str]] = None,
         dtype=None,
     ):
         """
@@ -39,9 +41,13 @@ class FieldCollection(FieldBase):
                 the individual fields given by `fields`.
             copy_fields (bool):
                 Flag determining whether the individual fields given in `fields`
-                are copied.
+                are copied. Note that fields are always copied if some of the supplied
+                fields are identical.
             label (str):
                 Label of the field collection
+            labels (list of str):
+                Labels of the individual fields. If omitted, the labels from the
+                `fields` argument are used.
             dtype (numpy dtype):
                 The data type of the field. All the numpy dtypes are supported. If
                 omitted, it will be determined from `data` automatically.
@@ -63,6 +69,12 @@ class FieldCollection(FieldBase):
             grids = [f.grid for f in fields]
             raise RuntimeError(f"Grids are incompatible: {grids}")
 
+        # check whether some fields are identical
+        if not copy_fields and len(fields) != len(set(id(field) for field in fields)):
+            self._logger = logging.getLogger(self.__class__.__name__)
+            self._logger.warning("Creating a copy of identical fields in collection")
+            copy_fields = True
+
         # create the list of underlying fields
         if copy_fields:
             self._fields = [field.copy() for field in fields]
@@ -112,6 +124,9 @@ class FieldCollection(FieldBase):
                 assert field.data.shape == field_shape
                 assert field.data.base is self.data
 
+        if labels is not None:
+            self.labels = labels  # type: ignore
+
     def __repr__(self):
         """ return instance as string """
         fields = []
@@ -536,7 +551,12 @@ class FieldCollection(FieldBase):
 
     @plot_on_figure(update_method="_update_plot")
     def plot(
-        self, kind: str = "auto", resize_fig: bool = True, fig=None, **kwargs
+        self,
+        kind: str = "auto",
+        resize_fig: bool = True,
+        fig=None,
+        subplot_args=None,
+        **kwargs,
     ) -> List[PlotReference]:
         r"""visualize all the fields in the collection
 
@@ -548,12 +568,16 @@ class FieldCollection(FieldBase):
             resize_fig (bool):
                 Whether to resize the figure to adjust to the number of panels
             {PLOT_ARGS}
+            subplot_args (list):
+                Additional arguments for the specific subplots. Should be a list with a
+                dictionary of arguments for each subplot. Supplying an empty allows to
+                keep the default setting of specific subplots.
             \**kwargs:
                 All additional keyword arguments are forwarded to the actual
-                plotting function.
+                plotting function of all subplots.
 
         Returns:
-            list of :class:`PlotReference`: Instances that contain information
+            List of :class:`PlotReference`: Instances that contain information
             to update all the plots with new data later.
         """
         # disable interactive plotting temporarily
@@ -562,10 +586,13 @@ class FieldCollection(FieldBase):
             fig.set_size_inches((4 * len(self), 3), forward=True)
         (axs,) = fig.subplots(1, len(self), squeeze=False)
 
-        # plot all the elements into the respective axes
+        if subplot_args is None:
+            subplot_args = [{}] * len(self)
+
+        # plot all the elements onto the respective axes
         reference = [
-            field.plot(kind=kind, ax=ax, action="create", **kwargs)
-            for field, ax in zip(self.fields, axs)
+            field.plot(kind=kind, ax=ax, action="create", **kwargs, **sp_args)
+            for field, ax, sp_args in zip(self.fields, axs, subplot_args)
         ]
 
         # return the references for all subplots
