@@ -35,12 +35,12 @@ from .common import (
 
 @fill_in_docstring
 def _get_laplace_matrix_1d(bcs) -> Tuple[Any, Any]:
-    """ get sparse matrix for laplace operator on a 1d Cartesian grid
-    
+    """get sparse matrix for laplace operator on a 1d Cartesian grid
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
         tuple: A sparse matrix and a sparse vector that can be used to evaluate
         the discretized laplacian
@@ -76,12 +76,12 @@ def _get_laplace_matrix_1d(bcs) -> Tuple[Any, Any]:
 
 @fill_in_docstring
 def _get_laplace_matrix_2d(bcs) -> Tuple[Any, Any]:
-    """ get sparse matrix for laplace operator on a 2d Cartesian grid
-    
+    """get sparse matrix for laplace operator on a 2d Cartesian grid
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
         tuple: A sparse matrix and a sparse vector that can be used to evaluate
         the discretized laplacian
@@ -94,8 +94,8 @@ def _get_laplace_matrix_2d(bcs) -> Tuple[Any, Any]:
     scale_x, scale_y = bcs.grid.discretization ** -2
 
     def i(x, y):
-        """ helper function for flattening the index
-        
+        """helper function for flattening the index
+
         This is equivalent to np.ravel_multi_index((x, y), (dim_x, dim_y))
         """
         return x * dim_y + y
@@ -144,12 +144,12 @@ def _get_laplace_matrix_2d(bcs) -> Tuple[Any, Any]:
 
 @fill_in_docstring
 def _get_laplace_matrix(bcs: Boundaries):
-    """ get sparse matrix for laplace operator on a 1d Cartesian grid
-    
+    """get sparse matrix for laplace operator on a 1d Cartesian grid
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
         tuple: A sparse matrix and a sparse vector that can be used to evaluate
         the discretized laplacian
@@ -172,8 +172,8 @@ def _get_laplace_matrix(bcs: Boundaries):
 def _make_derivative(
     bcs: Boundaries, axis: int = 0, method: str = "central"
 ) -> Callable:
-    """ make a derivative operator along a single axis using numba compilation
-    
+    """make a derivative operator along a single axis using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
@@ -182,7 +182,7 @@ def _make_derivative(
         method (str):
             The method for calculating the derivative. Possible values are
             'central', 'forward', and 'backward'.
-        
+
     Returns:
         A function that can be applied to an array of values. The result will be
         an array of the same shape containing the actual derivatives at the grid
@@ -257,14 +257,14 @@ def _make_derivative(
 
 @fill_in_docstring
 def _make_laplace_scipy_nd(bcs: Boundaries) -> Callable:
-    """ make a laplace operator using the scipy module
-    
+    """make a laplace operator using the scipy module
+
     This only supports uniform discretizations.
-    
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
         A function that can be applied to an array of values
     """
@@ -273,6 +273,7 @@ def _make_laplace_scipy_nd(bcs: Boundaries) -> Callable:
 
     def laplace(arr, out=None):
         """ apply laplace operator to array `arr` """
+        assert arr.shape == bcs.grid.shape
         return ndimage.laplace(scaling * arr, output=out, **args)
 
     return laplace
@@ -299,14 +300,14 @@ def _make_laplace_scipy_nd(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_laplace_numba_1d(bcs: Boundaries) -> Callable:
-    """ make a 1d laplace operator using numba compilation
-    
+    """make a 1d laplace operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     dim_x = bcs.grid.shape[0]
     scale = bcs.grid.discretization[0] ** -2
@@ -326,14 +327,14 @@ def _make_laplace_numba_1d(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_laplace_numba_2d(bcs: Boundaries) -> Callable:
-    """ make a 2d laplace operator using numba compilation
-    
+    """make a 2d laplace operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     dim_x, dim_y = bcs.grid.shape
     scale_x, scale_y = bcs.grid.discretization ** -2
@@ -385,12 +386,12 @@ def _make_laplace_numba_2d(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_laplace_numba_3d(bcs: Boundaries) -> Callable:
-    """ make a 3d laplace operator using numba compilation
-    
+    """make a 3d laplace operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
         A function that can be applied to an array of values
     """
@@ -428,14 +429,14 @@ def _make_laplace_numba_3d(bcs: Boundaries) -> Callable:
 @CartesianGridBase.register_operator("laplace", rank_in=0, rank_out=0)
 @fill_in_docstring
 def make_laplace(bcs: Boundaries, method: str = "auto") -> Callable:
-    """ make a laplace operator on a Cartesian grid
+    """make a laplace operator on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
         method (str): Method used for calculating the laplace operator.
             If method='auto', a suitable method is chosen automatically.
-        
+
     Returns:
         A function that can be applied to an array of values
     """
@@ -475,40 +476,44 @@ def make_laplace(bcs: Boundaries, method: str = "auto") -> Callable:
 
 @fill_in_docstring
 def _make_gradient_scipy_nd(bcs: Boundaries) -> Callable:
-    """ make a gradient operator using the scipy module
-    
+    """make a gradient operator using the scipy module
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
         A function that can be applied to an array of values
     """
-    scaling = 0.5 / bcs._uniform_discretization
+    scaling = 0.5 / bcs.grid.discretization
     dim = bcs.grid.dim
     shape_out = (dim,) + bcs.grid.shape
     args = bcs._scipy_border_mode
 
     def gradient(arr, out=None):
         """ apply gradient operator to array `arr` """
+        assert arr.shape == bcs.grid.shape
         if out is None:
             out = np.empty(shape_out)
+        else:
+            assert out.shape == shape_out
+
         for i in range(dim):
-            out[i] = ndimage.convolve1d(arr, [1, 0, -1], axis=i, **args)
-        return out * scaling
+            out[i] = ndimage.convolve1d(arr, [1, 0, -1], axis=i, **args) * scaling[i]
+        return out
 
     return gradient
 
 
 @fill_in_docstring
 def _make_gradient_numba_1d(bcs: Boundaries) -> Callable:
-    """ make a 1d gradient operator using numba compilation
-    
+    """make a 1d gradient operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
-    Returns:    
+
+    Returns:
         A function that can be applied to an array of values
     """
     dim_x = bcs.grid.shape[0]
@@ -529,14 +534,14 @@ def _make_gradient_numba_1d(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_gradient_numba_2d(bcs: Boundaries) -> Callable:
-    """ make a 2d gradient operator using numba compilation
-    
+    """make a 2d gradient operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     dim_x, dim_y = bcs.grid.shape
     scale_x, scale_y = 0.5 / bcs.grid.discretization
@@ -565,14 +570,14 @@ def _make_gradient_numba_2d(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_gradient_numba_3d(bcs: Boundaries) -> Callable:
-    """ make a 3d gradient operator using numba compilation
-    
+    """make a 3d gradient operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     dim_x, dim_y, dim_z = bcs.grid.shape
     scale_x, scale_y, scale_z = 0.5 / bcs.grid.discretization
@@ -606,14 +611,14 @@ def _make_gradient_numba_3d(bcs: Boundaries) -> Callable:
 @CartesianGridBase.register_operator("gradient", rank_in=0, rank_out=1)
 @fill_in_docstring
 def make_gradient(bcs: Boundaries, method: str = "auto") -> Callable:
-    """ make a gradient operator on a Cartesian grid
+    """make a gradient operator on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
         method (str): Method used for calculating the gradient operator.
             If method='auto', a suitable method is chosen automatically.
-        
+
     Returns:
         A function that can be applied to an array of values
     """
@@ -650,8 +655,8 @@ def make_gradient(bcs: Boundaries, method: str = "auto") -> Callable:
 
 @fill_in_docstring
 def _make_gradient_squared_numba_1d(bcs: Boundaries, central: bool = True) -> Callable:
-    """ make a 1d squared gradient operator using numba compilation
-    
+    """make a 1d squared gradient operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
@@ -660,8 +665,8 @@ def _make_gradient_squared_numba_1d(bcs: Boundaries, central: bool = True) -> Ca
             operator. If this is False, the squared gradient is calculated as
             the mean of the squared values of the forward and backward
             derivatives.
-        
-    Returns:    
+
+    Returns:
         A function that can be applied to an array of values
     """
     dim_x = bcs.grid.shape[0]
@@ -698,8 +703,8 @@ def _make_gradient_squared_numba_1d(bcs: Boundaries, central: bool = True) -> Ca
 
 @fill_in_docstring
 def _make_gradient_squared_numba_2d(bcs: Boundaries, central: bool = True) -> Callable:
-    """ make a 2d squared gradient operator using numba compilation
-    
+    """make a 2d squared gradient operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
@@ -708,9 +713,9 @@ def _make_gradient_squared_numba_2d(bcs: Boundaries, central: bool = True) -> Ca
             operator. If this is False, the squared gradient is calculated as
             the mean of the squared values of the forward and backward
             derivatives.
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     dim_x, dim_y = bcs.grid.shape
 
@@ -761,8 +766,8 @@ def _make_gradient_squared_numba_2d(bcs: Boundaries, central: bool = True) -> Ca
 
 @fill_in_docstring
 def _make_gradient_squared_numba_3d(bcs: Boundaries, central: bool = True) -> Callable:
-    """ make a 3d squared gradient operator using numba compilation
-    
+    """make a 3d squared gradient operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
@@ -771,9 +776,9 @@ def _make_gradient_squared_numba_3d(bcs: Boundaries, central: bool = True) -> Ca
             operator. If this is False, the squared gradient is calculated as
             the mean of the squared values of the forward and backward
             derivatives.
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     dim_x, dim_y, dim_z = bcs.grid.shape
 
@@ -835,7 +840,7 @@ def _make_gradient_squared_numba_3d(bcs: Boundaries, central: bool = True) -> Ca
 @CartesianGridBase.register_operator("gradient_squared", rank_in=0, rank_out=0)
 @fill_in_docstring
 def make_gradient_squared(bcs: Boundaries, central: bool = True) -> Callable:
-    """ make a gradient operator on a Cartesian grid
+    """make a gradient operator on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
@@ -845,7 +850,7 @@ def make_gradient_squared(bcs: Boundaries, central: bool = True) -> Callable:
             operator. If this is False, the squared gradient is calculated as
             the mean of the squared values of the forward and backward
             derivatives.
-        
+
     Returns:
         A function that can be applied to an array of values
     """
@@ -868,24 +873,23 @@ def make_gradient_squared(bcs: Boundaries, central: bool = True) -> Callable:
 
 @fill_in_docstring
 def _make_divergence_scipy_nd(bcs: Boundaries) -> Callable:
-    """ make a divergence operator using the scipy module
-    
-    This only supports uniform discretizations.
-    
+    """make a divergence operator using the scipy module
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
         A function that can be applied to an array of values
     """
     shape = bcs.grid.shape
-    scaling = 0.5 / bcs._uniform_discretization
+    scaling = 0.5 / bcs.grid.discretization
     args = bcs._scipy_border_mode
 
     def divergence(arr, out=None):
         """ apply divergence operator to array `arr` """
-        assert arr.shape[0] == len(shape) and len(arr.shape) == len(shape) + 1
+        assert arr.shape[0] == len(shape) and arr.shape[1:] == shape
+
         # need to initialize with zeros since data is added later
         if out is None:
             out = np.zeros(arr.shape[1:])
@@ -893,23 +897,23 @@ def _make_divergence_scipy_nd(bcs: Boundaries) -> Callable:
             out[:] = 0
 
         for i in range(len(shape)):
-            out += ndimage.convolve1d(arr[i], [1, 0, -1], axis=i, **args)
-        return out * scaling
+            out += ndimage.convolve1d(arr[i], [1, 0, -1], axis=i, **args) * scaling[i]
+        return out
 
     return divergence
 
 
 @fill_in_docstring
 def _make_divergence_numba_1d(bcs: Boundaries) -> Callable:
-    """ make a 1d divergence operator using numba compilation
-    
+    """make a 1d divergence operator using numba compilation
+
     Args:
         dim (int): The number of support points for each axes
         boundaries (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
         dx (float): The discretization
-        
-    Returns:    
+
+    Returns:
         A function that can be applied to an array of values
     """
     dim_x = bcs.grid.shape[0]
@@ -930,14 +934,14 @@ def _make_divergence_numba_1d(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_divergence_numba_2d(bcs: Boundaries) -> Callable:
-    """ make a 2d divergence operator using numba compilation
-    
+    """make a 2d divergence operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     dim_x, dim_y = bcs.grid.shape
     scale_x, scale_y = 0.5 / bcs.grid.discretization
@@ -967,14 +971,14 @@ def _make_divergence_numba_2d(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_divergence_numba_3d(bcs: Boundaries) -> Callable:
-    """ make a 3d divergence operator using numba compilation
-    
+    """make a 3d divergence operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     dim_x, dim_y, dim_z = bcs.grid.shape
     scale_x, scale_y, scale_z = 0.5 / bcs.grid.discretization
@@ -1009,14 +1013,14 @@ def _make_divergence_numba_3d(bcs: Boundaries) -> Callable:
 @CartesianGridBase.register_operator("divergence", rank_in=1, rank_out=0)
 @fill_in_docstring
 def make_divergence(bcs: Boundaries, method: str = "auto") -> Callable:
-    """ make a divergence operator on a Cartesian grid
+    """make a divergence operator on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
         method (str): Method used for calculating the divergence operator.
             If method='auto', a suitable method is chosen automatically.
-        
+
     Returns:
         A function that can be applied to an array of values
     """
@@ -1053,43 +1057,46 @@ def make_divergence(bcs: Boundaries, method: str = "auto") -> Callable:
 
 @fill_in_docstring
 def _make_vector_gradient_scipy_nd(bcs: Boundaries) -> Callable:
-    """ make a vector gradient operator using the scipy module
-    
-    This only supports uniform discretizations.
-    
+    """make a vector gradient operator using the scipy module
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
         A function that can be applied to an array of values
     """
-    scaling = 0.5 / bcs._uniform_discretization
+    scaling = 0.5 / bcs.grid.discretization
     args = bcs._scipy_border_mode
     dim = bcs.grid.dim
     shape_out = (dim, dim) + bcs.grid.shape
 
     def vector_gradient(arr, out=None):
         """ apply vector gradient operator to array `arr` """
+        assert arr.shape == shape_out[1:]
         if out is None:
             out = np.empty(shape_out)
+        else:
+            assert out.shape == shape_out
+
         for i in range(dim):
             for j in range(dim):
-                out[i, j] = ndimage.convolve1d(arr[j], [1, 0, -1], axis=i, **args)
-        return out * scaling
+                conv = ndimage.convolve1d(arr[j], [1, 0, -1], axis=i, **args)
+                out[i, j] = conv * scaling[i]
+        return out
 
     return vector_gradient
 
 
 @fill_in_docstring
 def _make_vector_gradient_numba_1d(bcs: Boundaries) -> Callable:
-    """ make a 1d vector gradient operator using numba compilation
-    
+    """make a 1d vector gradient operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
-    Returns:    
+
+    Returns:
         A function that can be applied to an array of values
     """
     gradient = _make_gradient_numba_1d(bcs.extract_component(0))
@@ -1105,14 +1112,14 @@ def _make_vector_gradient_numba_1d(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_vector_gradient_numba_2d(bcs: Boundaries) -> Callable:
-    """ make a 2d vector gradient operator using numba compilation
-    
+    """make a 2d vector gradient operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     gradient_x = _make_gradient_numba_2d(bcs.extract_component(0))
     gradient_y = _make_gradient_numba_2d(bcs.extract_component(1))
@@ -1129,16 +1136,16 @@ def _make_vector_gradient_numba_2d(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_vector_gradient_numba_3d(bcs: Boundaries) -> Callable:
-    """ make a 3d vector gradient operator using numba compilation
-    
+    """make a 3d vector gradient operator using numba compilation
+
     Args:
         shape (tuple): The number of support points for each axes
         boundaries (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
         dx (float or list): The discretizations
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     gradient_x = _make_gradient_numba_3d(bcs.extract_component(0))
     gradient_y = _make_gradient_numba_3d(bcs.extract_component(1))
@@ -1158,14 +1165,14 @@ def _make_vector_gradient_numba_3d(bcs: Boundaries) -> Callable:
 @CartesianGridBase.register_operator("vector_gradient", rank_in=1, rank_out=2)
 @fill_in_docstring
 def make_vector_gradient(bcs: Boundaries, method: str = "auto") -> Callable:
-    """ make a vector gradient operator on a Cartesian grid
+    """make a vector gradient operator on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
         method (str): Method used for calculating the vector gradient operator.
             If method='auto', a suitable method is chosen automatically
-        
+
     Returns:
         A function that can be applied to an array of values
     """
@@ -1201,14 +1208,14 @@ def make_vector_gradient(bcs: Boundaries, method: str = "auto") -> Callable:
 
 @fill_in_docstring
 def _make_vector_laplace_scipy_nd(bcs: Boundaries) -> Callable:
-    """ make a vector Laplacian using the scipy module
-    
+    """make a vector Laplacian using the scipy module
+
     This only supports uniform discretizations.
-    
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
         A function that can be applied to an array of values
     """
@@ -1219,8 +1226,12 @@ def _make_vector_laplace_scipy_nd(bcs: Boundaries) -> Callable:
 
     def vector_laplace(arr, out=None):
         """ apply vector Laplacian operator to array `arr` """
+        assert arr.shape == shape_out
         if out is None:
             out = np.empty(shape_out)
+        else:
+            assert out.shape == shape_out
+
         for i in range(dim):
             ndimage.laplace(arr[i], output=out[i], **args)
         return out * scaling
@@ -1230,13 +1241,13 @@ def _make_vector_laplace_scipy_nd(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_vector_laplace_numba_1d(bcs: Boundaries) -> Callable:
-    """ make a 1d vector Laplacian using numba compilation
-    
+    """make a 1d vector Laplacian using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
-    Returns:    
+
+    Returns:
         A function that can be applied to an array of values
     """
     laplace = _make_laplace_numba_1d(bcs.extract_component(0))
@@ -1252,14 +1263,14 @@ def _make_vector_laplace_numba_1d(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_vector_laplace_numba_2d(bcs: Boundaries) -> Callable:
-    """ make a 2d vector Laplacian using numba compilation
-    
+    """make a 2d vector Laplacian using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     laplace_x = _make_laplace_numba_2d(bcs.extract_component(0))
     laplace_y = _make_laplace_numba_2d(bcs.extract_component(1))
@@ -1276,14 +1287,14 @@ def _make_vector_laplace_numba_2d(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_vector_laplace_numba_3d(bcs: Boundaries) -> Callable:
-    """ make a 3d vector Laplacian using numba compilation
-    
+    """make a 3d vector Laplacian using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     laplace_x = _make_laplace_numba_3d(bcs.extract_component(0))
     laplace_y = _make_laplace_numba_3d(bcs.extract_component(1))
@@ -1303,14 +1314,14 @@ def _make_vector_laplace_numba_3d(bcs: Boundaries) -> Callable:
 @CartesianGridBase.register_operator("vector_laplace", rank_in=1, rank_out=1)
 @fill_in_docstring
 def make_vector_laplace(bcs: Boundaries, method: str = "auto") -> Callable:
-    """ make a vector Laplacian on a Cartesian grid
+    """make a vector Laplacian on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
         method (str): Method used for calculating the vector laplace operator.
             If method='auto', a suitable method is chosen automatically.
-        
+
     Returns:
         A function that can be applied to an array of values
     """
@@ -1346,18 +1357,16 @@ def make_vector_laplace(bcs: Boundaries, method: str = "auto") -> Callable:
 
 @fill_in_docstring
 def _make_tensor_divergence_scipy_nd(bcs: Boundaries) -> Callable:
-    """ make a tensor divergence operator using the scipy module
-    
-    This only supports uniform discretizations.
-    
+    """make a tensor divergence operator using the scipy module
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
         A function that can be applied to an array of values
     """
-    scaling = 0.5 / bcs._uniform_discretization
+    scaling = 0.5 / bcs.grid.discretization
     args = bcs._scipy_border_mode
     dim = bcs.grid.dim
     shape_out = (dim,) + bcs.grid.shape
@@ -1365,28 +1374,31 @@ def _make_tensor_divergence_scipy_nd(bcs: Boundaries) -> Callable:
     def tensor_divergence(arr, out=None):
         """ apply tensor divergence operator to array `arr` """
         # need to initialize with zeros since data is added later
+        assert arr.shape[0] == dim and arr.shape[1:] == shape_out
         if out is None:
             out = np.zeros(shape_out)
         else:
+            assert out.shape == shape_out
             out[:] = 0
 
         for i in range(dim):
             for j in range(dim):
-                out[i] += ndimage.convolve1d(arr[i, j], [1, 0, -1], axis=j, **args)
-        return out * scaling
+                conv = ndimage.convolve1d(arr[i, j], [1, 0, -1], axis=j, **args)
+                out[i] += conv * scaling[j]
+        return out
 
     return tensor_divergence
 
 
 @fill_in_docstring
 def _make_tensor_divergence_numba_1d(bcs: Boundaries) -> Callable:
-    """ make a 1d tensor divergence  operator using numba compilation
-    
+    """make a 1d tensor divergence  operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
-    Returns:    
+
+    Returns:
         A function that can be applied to an array of values
     """
     divergence = _make_divergence_numba_1d(bcs.extract_component(0))
@@ -1402,14 +1414,14 @@ def _make_tensor_divergence_numba_1d(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_tensor_divergence_numba_2d(bcs: Boundaries) -> Callable:
-    """ make a 2d tensor divergence  operator using numba compilation
-    
+    """make a 2d tensor divergence  operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     divergence_x = _make_divergence_numba_2d(bcs.extract_component(0))
     divergence_y = _make_divergence_numba_2d(bcs.extract_component(1))
@@ -1426,14 +1438,14 @@ def _make_tensor_divergence_numba_2d(bcs: Boundaries) -> Callable:
 
 @fill_in_docstring
 def _make_tensor_divergence_numba_3d(bcs: Boundaries) -> Callable:
-    """ make a 3d tensor divergence  operator using numba compilation
-    
+    """make a 3d tensor divergence  operator using numba compilation
+
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
             {ARG_BOUNDARIES_INSTANCE}
-        
+
     Returns:
-        A function that can be applied to an array of values    
+        A function that can be applied to an array of values
     """
     divergence_x = _make_divergence_numba_3d(bcs.extract_component(0))
     divergence_y = _make_divergence_numba_3d(bcs.extract_component(1))
@@ -1453,7 +1465,7 @@ def _make_tensor_divergence_numba_3d(bcs: Boundaries) -> Callable:
 @CartesianGridBase.register_operator("tensor_divergence", rank_in=2, rank_out=1)
 @fill_in_docstring
 def make_tensor_divergence(bcs: Boundaries, method: str = "auto") -> Callable:
-    """ make a tensor divergence operator on a Cartesian grid
+    """make a tensor divergence operator on a Cartesian grid
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
@@ -1461,7 +1473,7 @@ def make_tensor_divergence(bcs: Boundaries, method: str = "auto") -> Callable:
         method (str): Method used for calculating the tensor divergence
             operator. If method='auto', a suitable method is chosen
             automatically.
-        
+
     Returns:
         A function that can be applied to an array of values
     """
@@ -1498,7 +1510,7 @@ def make_tensor_divergence(bcs: Boundaries, method: str = "auto") -> Callable:
 @CartesianGridBase.register_operator("poisson_solver", rank_in=0, rank_out=0)
 @fill_in_docstring
 def make_poisson_solver(bcs: Boundaries, method: str = "auto") -> Callable:
-    """ make a operator that solves Poisson's equation
+    """make a operator that solves Poisson's equation
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
@@ -1506,7 +1518,7 @@ def make_poisson_solver(bcs: Boundaries, method: str = "auto") -> Callable:
         method (str): Method used for calculating the tensor divergence
             operator. If method='auto', a suitable method is chosen
             automatically.
-        
+
     Returns:
         A function that can be applied to an array of values
     """
