@@ -171,6 +171,9 @@ class StorageBase(metaclass=ABCMeta):
             :class:`~pde.fields.FieldBase`:
             The field class containing the grid and data
         """
+        if t_index < 0:
+            t_index += len(self)
+
         if not 0 <= t_index < len(self):
             raise IndexError("Time index out of range")
 
@@ -374,7 +377,7 @@ class StorageBase(metaclass=ABCMeta):
         """
         # get the number of arguments that the user function expects
         num_args = len(signature(func).parameters)
-        writing = False  # flag indicating whether output storage wass opened
+        writing = False  # flag indicating whether output storage was opened
 
         for t, field in display_progress(
             self.items(), total=len(self), enabled=progress
@@ -412,6 +415,25 @@ class StorageBase(metaclass=ABCMeta):
 
         return out
 
+    def copy(
+        self, out: "StorageBase" = None, *, progress: bool = False
+    ) -> "StorageBase":
+        """copies all fields in a storage to a new one
+
+        Args:
+            out (:class:`~pde.storage.base.StorageBase`):
+                Storage to which the output is written. If omitted, a new
+                :class:`~pde.storage.memory.MemoryStorage` is used and returned
+            progress (bool):
+                Flag indicating whether the progress is shown during the calculation
+
+        Returns:
+            :class:`~pde.storage.base.StorageBase`: The new storage that contains the
+            copied data
+        """
+        # apply the identity function to do the copy
+        return self.apply(lambda x: x, out=out, progress=progress)
+
 
 class StorageTracker(TrackerBase):
     """Tracker that stores data in special storage classes
@@ -456,7 +478,7 @@ class StorageTracker(TrackerBase):
                 The current state of the simulation
             t (float): The associated time
         """
-        self.storage.append(field.data, time=t)
+        self.storage.append(field, time=t)
 
     def finalize(self, info: InfoDict = None) -> None:
         """finalize the tracker, supplying additional information
