@@ -4,8 +4,7 @@
 
 import numpy as np
 import pytest
-
-from ..expressions import ScalarExpression, TensorExpression, parse_number
+from pde.tools.expressions import ScalarExpression, TensorExpression, parse_number
 
 
 def test_parse_number():
@@ -224,6 +223,19 @@ def test_expression_user_funcs():
     expr = ScalarExpression("func()", user_funcs={"func": lambda: 1})
     assert expr() == 1
     assert expr.get_compiled()() == 1
+    assert expr.value == 1
+
+    expr = ScalarExpression("f(pi)", user_funcs={"f": np.sin})
+    assert expr.constant
+    assert expr() == pytest.approx(0)
+    assert expr.get_compiled()() == pytest.approx(0)
+    assert expr.value == pytest.approx(0)
+
+    expr = TensorExpression("[0, f(pi)]", user_funcs={"f": np.sin})
+    assert expr.constant
+    np.testing.assert_allclose(expr(), np.array([0, 0]), atol=1e-14)
+    np.testing.assert_allclose(expr.get_compiled()(), np.array([0, 0]), atol=1e-14)
+    np.testing.assert_allclose(expr.value, np.array([0, 0]), atol=1e-14)
 
 
 def test_complex_expression():
@@ -247,3 +259,29 @@ def test_expression_special():
     assert expr(-1) == 0
     assert expr(0) == 0.5
     assert expr(1) == 1
+
+    f = expr.get_compiled()
+    assert f(-1) == 0
+    assert f(0) == 0.5
+    assert f(1) == 1
+
+
+def test_expression_consts():
+    """ test the usage of consts """
+    expr = ScalarExpression("a", consts={"a": 1})
+    assert expr.constant
+    assert not expr.depends_on("a")
+    assert expr() == 1
+    assert expr.get_compiled()() == 1
+    assert expr.value == 1
+
+    expr = ScalarExpression("a + b", consts={"a": 1})
+    assert not expr.constant
+    assert not expr.depends_on("a") and expr.depends_on("b")
+    assert expr(2) == 3
+    assert expr.get_compiled()(2) == 3
+
+    expr = ScalarExpression("a + b", consts={"a": np.array([1, 2])})
+    assert not expr.constant
+    np.testing.assert_allclose(expr(np.array([2, 3])), np.array([3, 5]))
+    np.testing.assert_allclose(expr.get_compiled()(np.array([2, 3])), np.array([3, 5]))
